/***************************************************************************
 *   Copyright (C) 2014 by Jan Stückrath <jan.stueckrath@uni-due.de>       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA 02110, USA                    *
 ***************************************************************************/

#ifndef GTXLERRORHANDLER_H_
#define GTXLERRORHANDLER_H_

#include <xercesc/sax/ErrorHandler.hpp>
#include <xercesc/sax/SAXParseException.hpp>

namespace uncover {
	namespace io {

		/**
		 * Prints errors generated by the Xerces framework. The errors are streamed by the standard
		 * logging system, preserving the error type.
		 * @author Jan Stückrath
		 */
		class XercesLoggingErrorHandler: public xercesc::ErrorHandler {

			public:

				/**
				 * Generates a new XerxesLoggingErrorHandler.
				 */
				XercesLoggingErrorHandler();

				/**
				 * Destroys this XerxesLoggingErrorHandler.
				 */
				virtual ~XercesLoggingErrorHandler();

				/**
				 * Prints a Xerxes warning to the standard logging stream.
				 * @param exc the exception generated by Xerces
				 */
				virtual void warning (const xercesc::SAXParseException &exc) override;

				/**
				 * Prints a Xerxes error to the standard logging stream.
				 * @param exc the exception generated by Xerces
				 */
				virtual void error (const xercesc::SAXParseException &exc) override;

				/**
				 * Prints a Xerxes fatal error to the standard logging stream.
				 * @param exc the exception generated by Xerces
				 */
				virtual void fatalError (const xercesc::SAXParseException &exc) override;

				/**
				 * Since errors are not stores by this class, this function does nothing
				 */
				virtual void resetErrors () override;

		};

	} /* namespace io */
} /* namespace uncover */

#endif /* GTXLERRORHANDLER_H_ */
